#nvpl_sparse-config.cmake
#
# Imported interface targets provided:
#  * nvpl::sparse_static - static library target (if available)
#  * nvpl::sparse_shared - shared library target
#  * nvpl::sparse - alias to shared library target
#
# Output variables set:
#  * nvpl_sparse_VERSION - Version of installed package
#  * nvpl_sparse_INCLUDE_DIR - Location of nvpl_sparse headers
#  * nvpl_sparse_LIBRARY_DIR - Location of nvpl_sparse libraries

set(nvpl_sparse_VERSION "0.4.0")
# build: 160


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was PackageConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

## Dependencies
include(CMakeFindDependencyMacro)

## Find header and lib directories via known header and lib names
set(_nvpl_sparse_search_header_name nvpl_sparse.h)
set(_nvpl_sparse_search_lib_name libnvpl_sparse.so)
get_filename_component(_nvpl_sparse_search_prefix "${CMAKE_CURRENT_LIST_DIR}/../../" ABSOLUTE)

find_path(nvpl_sparse_INCLUDE_DIR
          NAMES ${_nvpl_sparse_search_header_name}
          PATHS ${_nvpl_sparse_search_prefix}
          PATH_SUFFIXES
            ../include
            ../../include # Debian/Ubuntu aarch64
          NO_DEFAULT_PATH
          NO_CACHE) # Set cache variable after path normalization

find_path(nvpl_sparse_LIBRARY_DIR
          NAMES ${_nvpl_sparse_search_lib_name}
          PATHS ${_nvpl_sparse_search_prefix}
          PATH_SUFFIXES lib lib64
          DOC "Location of nvpl_sparse libraries"
          NO_DEFAULT_PATH)

# Check headers and library directories are found
if(NOT EXISTS "${nvpl_sparse_INCLUDE_DIR}")
    message(FATAL_ERROR "Header directory containing file ${_nvpl_sparse_search_header_name} was not found relative to ${CMAKE_CURRENT_LIST_DIR}!")
endif()
if(NOT EXISTS "${nvpl_sparse_LIBRARY_DIR}")
    message(FATAL_ERROR "Library directory containing file ${_nvpl_sparse_search_lib_name} was not found relative to ${CMAKE_CURRENT_LIST_DIR}!")
endif()

# Normalize nvpl_sparse_INCLUDE_DIR and set cache variable
get_filename_component(nvpl_sparse_INCLUDE_DIR "${nvpl_sparse_INCLUDE_DIR}" ABSOLUTE)
set(nvpl_sparse_INCLUDE_DIR ${nvpl_sparse_INCLUDE_DIR} CACHE STRING "Location of nvpl_sparse headers")

unset(_nvpl_sparse_search_header_name)
unset(_nvpl_sparse_search_lib_name)
unset(_nvpl_sparse_search_prefix)

## Targets
if(NOT TARGET nvpl_sparse)
  include("${CMAKE_CURRENT_LIST_DIR}/nvpl_sparse-targets.cmake")
  include("${CMAKE_CURRENT_LIST_DIR}/nvpl_sparse-static-targets.cmake" OPTIONAL)
endif()

if(TARGET nvpl_sparse_static)
  set(nvpl_sparse_STATIC_LIBRARIES nvpl_sparse_static)
endif()
set(nvpl_sparse_SHARED_LIBRARIES nvpl_sparse)

if(NOT TARGET nvpl::sparse)
  set_target_properties(nvpl_sparse PROPERTIES IMPORTED_GLOBAL 1)
  add_library(nvpl::sparse ALIAS nvpl_sparse)
endif()
if(NOT TARGET nvpl::sparse_shared)
  add_library(nvpl::sparse_shared ALIAS nvpl_sparse)
endif()
if(TARGET nvpl_sparse_static AND NOT TARGET nvpl::sparse_static)
  set_target_properties(nvpl_sparse_static PROPERTIES IMPORTED_GLOBAL 1)
  add_library(nvpl::sparse_static ALIAS nvpl_sparse_static)
endif()

check_required_components(nvpl_sparse)
if(NOT ${CMAKE_FIND_PACKAGE_NAME}_FIND_QUIETLY)
  message(STATUS "Found nvpl_sparse: (Version:${nvpl_sparse_VERSION}
                       CMakePackageDir:${CMAKE_CURRENT_LIST_DIR}
                       IncludeDir:${nvpl_sparse_INCLUDE_DIR}
                       LibraryDir:${nvpl_sparse_LIBRARY_DIR})")
endif()
